==> xfxwidgets-1.8.43.tar.gz: lib/XF_GRID/XElGrid.dart <==
import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_core/theme.dart';
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:xcommonwidgets/xcommonwidgets.dart';
import 'package:xfxbase/XFDataItem.dart';
import 'package:xfxbase/_XFX/XF_View.dart';
import 'package:xfxbase/_XFX/XFx.dart';
import 'package:xtoolspackage/xtoolspackage.dart';

class XElGrid<T, K extends XFDataItem> extends StatefulWidget {
  final String title;
  final XView view;
  final List<Widget>? yourActions;
  final Widget? titleWidget;
  final Widget? subTitleWidget;
  late Widget? xDrawerMenu;

  String? subTitle;
  bool allowDelRow;
  bool allowEdit;
  bool allowSwipingRow;
  bool allowSort;
  bool allowTriStateSort;
  bool allowMultiSort;
  bool allowColumnsResizing;
  final List<Widget>? endSwipeActionsBuilder;
  String Function(K)? youtText_DelAlert;
  Function(K)? funct_DelAlert;

  XElGrid(this.title, this.view, {Key? key, this.xDrawerMenu, this.yourActions, this.titleWidget, this.subTitleWidget, this.subTitle, this.allowDelRow = true, this.allowEdit = false, this.allowSwipingRow = true, this.allowSort = true, this.allowTriStateSort = true, this.allowMultiSort = true, this.allowColumnsResizing = true, this.youtText_DelAlert, this.funct_DelAlert, this.endSwipeActionsBuilder}) : super(key: key);

  @override
  State<XElGrid<T, K>> createState() => XElGridState<T, K>();
}

// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
class XElGridState<T, K extends XFDataItem> extends State<XElGrid<T, K>> {
  // XElencoState() : super() {}
  //XElencoState(this.xgds) : super() {}

  bool isLoading = false;
  List<K> listUI = List<K>.empty(growable: true);
  late XGridDataSource<K>? Function() xgds;
  late Future<List<K>?> Function() xLoadData_apiCall;

  xEditItem(BuildContext context, K item) async {}

  XGridDataSource<K>? xDSource;
  late DataGridController dataGridController;
  @override
  void initState() {
    dataGridController = DataGridController();
    xgds = xLoadData_getXGDS;
    xElenco_InitData();
    super.initState();
  }

  xLoadDataOnly() {
    xLoadData_apiCall().then((value) {
      listUI.clear();
      listUI.addAll(value!);
    }).whenComplete(() {
      setState(() {
        xDSource = xgds();
      });
    });
  }

  List<Widget>? xBuildYour_Actions() {
    return [Container()];
  }

  Widget xBuildYour_EndSwipeAction(BuildContext context, DataGridRow dataGridRow, int index) {
    if (widget.endSwipeActionsBuilder == null) {
      return Container();
    } else {
      return Row(children: widget.endSwipeActionsBuilder!.map((e) => e).toList());
    }
  }

  late IconData titleIcon;
  late Color titleColor;
  late String titleSubText;
  Widget xBuildTitleWidget() {
    return widget.titleWidget == null
        // ? Row(children: [
        //     Text(widget.title + "  - ", style: XStyles.xStyleText(colorText: Colors.white, activeBold: true)),
        //     SizedBox(width: 5),
        //   ])
        ? //
        Row(children: [
            Container(margin: EdgeInsets.only(right: 10), child: Icon(titleIcon, color: titleColor, size: 50)),
            Text(widget.title + "  - ", style: XStyles.xStyleText(colorText: titleColor, activeBold: true)),
            SizedBox(width: 5),
            Text(titleSubText, style: XStyles.xStyleText(colorText: Colors.white, activeBold: true)),
          ])
        : widget.titleWidget ?? Container();
  }

  ///Setta tu xDSource e questo metodo viene fatto all'interno della Build
  xBuild_InitData() {}

  ///Viene fatto solo all'initstate dell'elenco
  xElenco_InitData() {
    xLoadDataOnly();
  }

  XGridDataSource<K> xLoadData_getXGDS() {
    return XGridDataSource<K>([], XView(""));
  }

  Widget xBuildPageLayout(BuildContext context) {
    return GestureDetector(
        onTap: () {
          FocusScope.of(context).unfocus();
        },
        child: widget.xDrawerMenu != null
            ? Scaffold(
                appBar: AppBar(
                  iconTheme: IconThemeData(color: Colors.white),
                  actions: xBuildYour_Actions(),
                  title: xBuildTitleWidget(),
                ),
                body: Container(
                    margin: EdgeInsets.symmetric(horizontal: 15),
                    child: Column(children: [
                      SizedBox(height: 10),
                      Row(children: [Expanded(child: widget.subTitleWidget ?? XContainerWithLabel(widget.subTitle ?? widget.title, height: 30, color: Colors.black, textStyle: XStyles.xStyTextForTitle()))]),
                      SizedBox(height: 3),
                      xDSource == null
                          ? Container(child: Text("no items")) // questo serve se no fa il flash rosso e poi carica bene
                          : xBuilder_XGrid(context, xDSource!),
                    ])),
                drawer: XDrawer(logOut_OnTap: () {}, logOut_Visible: false, title: "", menu_Child: widget.xDrawerMenu),
              )
            : Container(
                margin: EdgeInsets.symmetric(horizontal: 15),
                child: Column(children: [
                  SizedBox(height: 10),
                  widget.subTitle != null || widget.subTitleWidget != null ? Row(children: [Expanded(child: widget.subTitleWidget ?? XContainerWithLabel(widget.subTitle ?? widget.title, height: 30, color: Colors.black, textStyle: XStyles.xStyTextForTitle()))]) : Container(),
                  SizedBox(height: 3),
                  xDSource == null
                      ? Container(child: Text("no items")) // questo serve se no fa il flash rosso e poi carica bene
                      : xBuilder_XGrid(context, xDSource!),
                ])));
  }

  xOnPressedDelete(K item, BuildContext context, {String? title_Text, Function(K)? func}) async {
    var r = await (xBuild_PopUp(context, item, "Are you sure to DELETE ${title_Text ?? "<No TITLE>"}"));
    print("result alert base" + r.toString());
    if (r!) {
      func!(item);
    }
  }

  Widget xBuilder_XGrid(BuildContext context, XGridDataSource dsource) {
    return Expanded(
        child: Container(
            child: SfDataGridTheme(
                data: SfDataGridThemeData(
                  // headerHoverColor: Colors.white.withOpacity(0.3),
                  // headerColor: Colors.white,

                  sortIconColor: ThemeData.estimateBrightnessForColor(Colors.black) == Brightness.dark ? Colors.white : Colors.black,
                  gridLineColor: Colors.white,
                  gridLineStrokeWidth: 2,
                  headerColor: Colors.grey[900],
                  selectionColor: Colors.grey[800],
                  currentCellStyle: DataGridCurrentCellStyle(borderColor: Colors.white, borderWidth: 1),
                ),
                child: SfDataGrid(
                  source: dsource,
                  isScrollbarAlwaysShown: true,
                  onCellTap: (details) {
                    if (dataGridController.selectedRow != null) {
                      if (details.rowColumnIndex.rowIndex - 1 == dataGridController.selectedIndex) {
                        xEditItem(context, (dataGridController.selectedRow as XDataGridRow).item as K);
                      }
                    }
                  },
                  onSelectionChanged: (addedRows, removedRows) {
                    if (addedRows.isNotEmpty) {
                      xEditItem(context, (dataGridController.selectedRow as XDataGridRow).item as K);
                    }
                  },
                  endSwipeActionsBuilder: (context, dataGridRow, index) {
                    return Row(mainAxisAlignment: MainAxisAlignment.center, children: [
                      xBuildYour_EndSwipeAction(context, dataGridRow, index),
                      widget.allowDelRow
                          ? Container(
                              width: 87,
                              alignment: Alignment.center,
                              margin: EdgeInsets.only(right: 10),
                              child: ElevatedButton(
                                style: ElevatedButton.styleFrom(backgroundColor: Colors.redAccent),
                                child: Text("Remove", textAlign: TextAlign.center, style: TextStyle(color: Colors.white)),
                                onPressed: () {
                                  xOnPressedDelete(
                                    (dataGridRow as XDataGridRow).item as K,
                                    context,
                                    title_Text: widget.youtText_DelAlert!(dataGridRow.item as K),
                                    func: widget.funct_DelAlert != null
                                        ? (value) {
                                            widget.funct_DelAlert!(value);
                                            // xDSource!.dataGridRows.remove(dataGridRow);
                                            // xDSource!.list.remove(dataGridRow.item);
                                          }
                                        : (value) {},
                                  );
                                },
                              ),
                            )
                          : Container()
                    ]);
                  },
                  selectionMode: SelectionMode.single,
                  navigationMode: GridNavigationMode.cell,
                  columnWidthMode: ColumnWidthMode.fill,
                  headerGridLinesVisibility: GridLinesVisibility.both,
                  gridLinesVisibility: GridLinesVisibility.both,
                  allowEditing: widget.allowEdit,
                  allowSwiping: widget.allowDelRow ? true : widget.allowSwipingRow,
                  allowSorting: widget.allowSort,
                  allowFiltering: true,
                  allowMultiColumnSorting: widget.allowMultiSort,
                  allowTriStateSorting: widget.allowTriStateSort,
                  allowColumnsResizing: widget.allowColumnsResizing,
                  columnWidthCalculationRange: ColumnWidthCalculationRange.allRows,
                  onColumnResizeUpdate: (details) {
                    var x = widget.view.cols.firstWhere((element) => element.colKey == details.column.columnName);
                    setState(() => x.width = details.width);
                    return true;
                  },
                  controller: dataGridController,
                  columns: widget.view.cols.map((e) => e.getXGCol()).toList(),
                ))));
  }

  @override
  Widget build(BuildContext context) {
    return xBuildPageLayout(context);
  }

  Future<bool?> xBuild_PopUp(BuildContext context, K item, String textTitle) async {
    return showDialog(
        context: context,
        builder: (BuildContext context) {
          return AlertDialog(
              title: Center(child: Container(child: Text(textTitle, style: XStyles.xStyTextForTextBase()))),
              content: Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: <Widget>[
                      Expanded(
                          child: TextButton(
                              style: TextButton.styleFrom(backgroundColor: Colors.black),
                              child: Text('YES', style: TextStyle(color: Colors.greenAccent)),
                              onPressed: () {
                                Navigator.pop(context, true);
                              })),
                      SizedBox(width: 15),
                      Expanded(
                          child: TextButton(
                        style: TextButton.styleFrom(backgroundColor: Colors.black),
                        child: Text('NO', style: TextStyle(color: Colors.redAccent)),
                        onPressed: () {
                          Navigator.pop(context, false);
                        },
                      )),
                    ],
                  ),
                ],
              ));
        });
  }
}

==> xfxwidgets-1.9.99.tar.gz: lib/XF_GRID/XElGrid.dart <==
import 'package:flutter/material.dart';
import 'package:syncfusion_flutter_core/theme.dart';
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:xcommonwidgets/xcommonwidgets.dart';
import 'package:xfxbase/XFDataItem.dart';
import 'package:xfxbase/_XFX/XF_View.dart';
import 'package:xfxbase/_XFX/XFx.dart';
import 'package:xtoolspackage/xtoolspackage.dart';

class XElGrid<T, K extends XFDataItem> extends StatefulWidget {
  final String title;
  final XView view;
  final List<Widget>? yourActions;
  final Widget? titleWidget;
  final Widget? subTitleWidget;
  late Widget? xDrawerMenu;

  String? subTitle;
  bool allowDelRow;
  bool allowEdit;
  bool allowSwipingRow;
  bool allowSort;
  bool allowTriStateSort;
  bool allowMultiSort;
  bool allowColumnsResizing;
  final List<Widget>? endSwipeActionsBuilder;
  String Function(K)? youtText_DelAlert;
  Function(K)? funct_DelAlert;
  late List<K>? liToUse;
  XGridDataSource<K>? Function()? xgds_Origin;

  XElGrid(
    this.title,
    this.view, {
    Key? key,
    this.xDrawerMenu,
    this.yourActions,
    this.titleWidget,
    this.subTitleWidget,
    this.subTitle,
    this.allowDelRow = true,
    this.allowEdit = false,
    this.allowSwipingRow = true,
    this.allowSort = true,
    this.allowTriStateSort = true,
    this.allowMultiSort = true,
    this.allowColumnsResizing = true,
    this.youtText_DelAlert,
    this.funct_DelAlert,
    this.endSwipeActionsBuilder,
    this.liToUse,
    this.xgds_Origin,
  }) : super(key: key);

  @override
  State<XElGrid<T, K>> createState() => XElGridState<T, K>();
}

// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
// STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE   STATE
class XElGridState<T, K extends XFDataItem> extends State<XElGrid<T, K>> {
  // XElencoState() : super() {}
  //XElencoState(this.xgds) : super() {}

  bool isLoading = false;
  List<K> listUI = List<K>.empty(growable: true);
  late XGridDataSource<K>? Function() xgds;
  late Future<List<K>?> Function()? xLoadData_apiCall;

  xEditItem(BuildContext context, K item) async {}

  XGridDataSource<K>? xDSource;
  late DataGridController dataGridController;
  @override
  void initState() {
    dataGridController = DataGridController();
    xgds = xLoadData_getXGDS;
    widget.xgds_Origin != null ? xgds = () => widget.xgds_Origin!() : null;
    xElenco_InitData();
    super.initState();
  }

  xLoadDataOnly() async {
    if (xLoadData_apiCall == null && widget.xgds_Origin != null) {
      widget.xgds_Origin != null ? xgds = () => widget.xgds_Origin!() : null;
      // xLoadData_apiCall = widget.liToUse;
      listUI.clear();
      listUI.addAll(xgds()!.list);
      setState(() {
        xDSource = xgds();
      });
    } else {
      xLoadData_apiCall!().then((value) {
        listUI.clear();
        listUI.addAll(value!);
      }).whenComplete(() {
        setState(() {
          xDSource = xgds();
        });
      });
    }
  }

  List<Widget>? xBuildYour_Actions() {
    return [Container()];
  }

  Widget xBuildYour_EndSwipeAction(BuildContext context, DataGridRow dataGridRow, int index) {
    if (widget.endSwipeActionsBuilder == null) {
      return Container();
    } else {
      return Row(children: widget.endSwipeActionsBuilder!.map((e) => e).toList());
    }
  }

  late IconData titleIcon;
  late Color titleColor;
  late String titleSubText;
  Widget xBuildTitleWidget() {
    return widget.titleWidget == null
        // ? Row(children: [
        //     Text(widget.title + "  - ", style: XStyles.xStyleText(colorText: Colors.white, activeBold: true)),
        //     SizedBox(width: 5),
        //   ])
        ? //
        Row(children: [
            Container(margin: EdgeInsets.only(right: 10), child: Icon(titleIcon, color: titleColor, size: 50)),
            Text(widget.title + "  - ", style: XStyles.xStyleText(colorText: titleColor, activeBold: true)),
            SizedBox(width: 5),
            Text(titleSubText, style: XStyles.xStyleText(colorText: Colors.white, activeBold: true)),
          ])
        : widget.titleWidget ?? Container();
  }

  ///Setta tu xDSource e questo metodo viene fatto all'interno della Build
  xBuild_InitData() {}

  ///Viene fatto solo all'initstate dell'elenco
  xElenco_InitData() {
    xLoadDataOnly();
  }

  XGridDataSource<K> xLoadData_getXGDS() {
    return XGridDataSource<K>([], XView(""));
  }

  Widget xBuildPageLayout(BuildContext context) {
    return GestureDetector(
        onTap: () {
          FocusScope.of(context).unfocus();
        },
        child: widget.xDrawerMenu != null
            ? Scaffold(
                appBar: AppBar(
                  iconTheme: IconThemeData(color: Colors.white),
                  actions: xBuildYour_Actions(),
                  title: xBuildTitleWidget(),
                ),
                body: Container(
                    margin: EdgeInsets.symmetric(horizontal: 15),
                    child: Column(children: [
                      SizedBox(height: 10),
                      Row(children: [Expanded(child: widget.subTitleWidget ?? XContainerWithLabel(widget.subTitle ?? widget.title, height: 30, color: Colors.black, textStyle: XStyles.xStyTextForTitle()))]),
                      SizedBox(height: 3),
                      xDSource == null
                          ? Container(child: Text("no items")) // questo serve se no fa il flash rosso e poi carica bene
                          : xBuilder_XGrid(context, xDSource!),
                    ])),
                drawer: XDrawer(logOut_OnTap: () {}, logOut_Visible: false, title: "", menu_Child: widget.xDrawerMenu),
              )
            : Container(
                margin: EdgeInsets.symmetric(horizontal: 15),
                child: Column(children: [
                  SizedBox(height: 10),
                  widget.subTitle != null || widget.subTitleWidget != null ? Row(children: [Expanded(child: widget.subTitleWidget ?? XContainerWithLabel(widget.subTitle ?? widget.title, height: 30, color: Colors.black, textStyle: XStyles.xStyTextForTitle()))]) : Container(),
                  SizedBox(height: 3),
                  xDSource == null
                      ? Container(child: Text("no items")) // questo serve se no fa il flash rosso e poi carica bene
                      : xBuilder_XGrid(context, xDSource!),
                ])));
  }

  xOnPressedDelete(K item, BuildContext context, {String? title_Text, Function(K)? func}) async {
    var r = await (xBuild_PopUp(context, item, "Are you sure to DELETE ${title_Text ?? "<No TITLE>"}"));
    print("result alert base" + r.toString());
    if (r!) {
      func!(item);
    }
  }

  Widget xBuilder_XGrid(BuildContext context, XGridDataSource dsource) {
    return Expanded(
        child: Container(
            child: SfDataGridTheme(
                data: SfDataGridThemeData(
                  // headerHoverColor: Colors.white.withOpacity(0.3),
                  // headerColor: Colors.white,

                  sortIconColor: ThemeData.estimateBrightnessForColor(Colors.black) == Brightness.dark ? Colors.white : Colors.black,
                  gridLineColor: Colors.white,
                  gridLineStrokeWidth: 2,
                  headerColor: Colors.grey[900],
                  selectionColor: Colors.grey[800],
                  currentCellStyle: DataGridCurrentCellStyle(borderColor: Colors.white, borderWidth: 1),
                ),
                child: SfDataGrid(
                  source: dsource,
                  isScrollbarAlwaysShown: true,
                  onCellTap: (details) {
                    if (dataGridController.selectedRow != null) {
                      if (details.rowColumnIndex.rowIndex - 1 == dataGridController.selectedIndex) {
                        xEditItem(context, (dataGridController.selectedRow as XDataGridRow).item as K);
                      }
                    }
                  },
                  onSelectionChanged: (addedRows, removedRows) {
                    if (addedRows.isNotEmpty) {
                      xEditItem(context, (dataGridController.selectedRow as XDataGridRow).item as K);
                    }
                  },
                  endSwipeActionsBuilder: (context, dataGridRow, index) {
                    return Row(mainAxisAlignment: MainAxisAlignment.center, children: [
                      xBuildYour_EndSwipeAction(context, dataGridRow, index),
                      widget.allowDelRow
                          ? Container(
                              width: 87,
                              alignment: Alignment.center,
                              margin: EdgeInsets.only(right: 10),
                              child: ElevatedButton(
                                  style: ElevatedButton.styleFrom(backgroundColor: Colors.redAccent),
                                  child: Text("Remove", textAlign: TextAlign.center, style: TextStyle(color: Colors.white)),
                                  onPressed: () {
                                    xOnPressedDelete(
                                      (dataGridRow as XDataGridRow).item as K,
                                      context,
                                      title_Text: widget.youtText_DelAlert!(dataGridRow.item as K),
                                      func: widget.funct_DelAlert != null
                                          ? (value) {
                                              widget.funct_DelAlert!(value);
                                            }
                                          : (value) {},
                                    );
                                  }))
                          : Container()
                    ]);
                  },
                  selectionMode: SelectionMode.single,
                  navigationMode: GridNavigationMode.cell,
                  columnWidthMode: ColumnWidthMode.fill,
                  headerGridLinesVisibility: GridLinesVisibility.both,
                  gridLinesVisibility: GridLinesVisibility.both,
                  allowEditing: widget.allowEdit,
                  allowSwiping: widget.allowDelRow ? true : widget.allowSwipingRow,
                  allowSorting: widget.allowSort,
                  allowFiltering: true,
                  allowMultiColumnSorting: widget.allowMultiSort,
                  allowTriStateSorting: widget.allowTriStateSort,
                  allowColumnsResizing: widget.allowColumnsResizing,
                  columnWidthCalculationRange: ColumnWidthCalculationRange.allRows,
                  onColumnResizeUpdate: (details) {
                    var x = widget.view.cols.firstWhere((element) => element.colKey == details.column.columnName);
                    setState(() => x.width = details.width);
                    return true;
                  },
                  controller: dataGridController,
                  columns: widget.view.cols.map((e) => e.getXGCol()).toList(),
                ))));
  }

  @override
  Widget build(BuildContext context) {
    return xBuildPageLayout(context);
  }

  Future<bool?> xBuild_PopUp(BuildContext context, K item, String textTitle) async {
    return showDialog(
        context: context,
        builder: (BuildContext context) {
          return AlertDialog(
              title: Center(child: Container(child: Text(textTitle, style: XStyles.xStyTextForTextBase()))),
              content: Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: <Widget>[
                      Expanded(
                          child: TextButton(
                              style: TextButton.styleFrom(backgroundColor: Colors.black),
                              child: Text('YES', style: TextStyle(color: Colors.greenAccent)),
                              onPressed: () {
                                Navigator.pop(context, true);
                              })),
                      SizedBox(width: 15),
                      Expanded(
                          child: TextButton(
                        style: TextButton.styleFrom(backgroundColor: Colors.black),
                        child: Text('NO', style: TextStyle(color: Colors.redAccent)),
                        onPressed: () {
                          Navigator.pop(context, false);
                        },
                      )),
                    ],
                  ),
                ],
              ));
        });
  }
}
